@@ -1,7 +1,6 @@
 # Copyright 2016, Yahoo Inc.
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
-"""
-Generic utilities, or :mod:`.plot` API but without polluting imports.
+"""Generic utilities, exceptions and :term:`operation` & :term:`plottable` base classes.
 
 .. doctest::
     :hide:
@@ -16,9 +15,17 @@ import abc
 import inspect
 import logging
 from collections import abc as cabc
-from collections import defaultdict, namedtuple
 from functools import partial, partialmethod, wraps
-from typing import Any, Collection, List, Mapping, NamedTuple, Optional, Tuple, Union
+from typing import (
+    Any,
+    Callable,
+    Collection,
+    Mapping,
+    NamedTuple,
+    Optional,
+    Tuple,
+    Union,
+)
 
 Items = Union[Collection, str, None]
 
@@ -41,7 +48,7 @@ class AbortedException(Exception):
 
 class IncompleteExecutionError(Exception):
     """
-    Error report when any `netop` operations were canceled/failed.
+    Reported when any :term:`endured`/:term:`reschedule` operations were are canceled.
 
     The exception contains 3 arguments:
 
@@ -96,15 +103,16 @@ class Token(str):
         return self.__str__()
 
 
-#: When an operation function returns this special value,
-#: it implies operation has no result at all,
-#: (otherwise, it would have been a single result, ``None``).
-#: Usefull for :term:`partial outputs` who want to cancel their single result
-#: witout being delcared as :term:`returns dictionary`.
-NO_RESULT = Token("NO_RESULT")
 UNSET = Token("UNSET")
 
 
+def first_solid(*tristates, default=None):
+    """Utility combining multiple tri-state booleans."""
+    from boltons.iterutils import first
+
+    return first(tristates, default=default, key=lambda i: i is not None)
+
+
 def aslist(i, argname, allowed_types=list):
     """Utility to accept singular strings as lists, and None --> []."""
     if not i:
@@ -171,10 +179,10 @@ def func_name(
         'func_name'
         >>> func_name(func_name, mod=1)
         'graphtik.base.func_name'
-        >>> func_name(MultiValueError.mro, fqdn=0)
-        'mro'
-        >>> func_name(MultiValueError.mro, fqdn=1)
-        'MultiValueError.mro'
+        >>> func_name(func_name.__format__, fqdn=0)
+        '__format__'
+        >>> func_name(func_name.__format__, fqdn=1)
+        'function.__format__'
 
     Even functions defined in docstrings are reported:
 
@@ -527,7 +535,7 @@ class Plottable(abc.ABC):
             the :term:`plottable` that ordered the plotting.
             Automatically set downstreams to one of::
 
-                op | netop | net | plan | solution | <missing>
+                op | pipeline | net | plan | solution | <missing>
 
             :seealso: :attr:`.PlotArgs.plottable`
         :param plotter:
@@ -667,28 +675,38 @@ class Plottable(abc.ABC):
 
         **Examples:**
 
-        >>> from graphtik import compose, operation
-        >>> from graphtik.modifiers import optional
-        >>> from operator import add
+            >>> from graphtik import compose, operation
+            >>> from graphtik.modifiers import optional
+            >>> from operator import add
 
-        >>> netop = compose("netop",
-        ...     operation(name="add", needs=["a", "b1"], provides=["ab1"])(add),
-        ...     operation(name="sub", needs=["a", optional("b2")], provides=["ab2"])(lambda a, b=1: a-b),
-        ...     operation(name="abb", needs=["ab1", "ab2"], provides=["asked"])(add),
-        ... )
+            >>> pipeline = compose("pipeline",
+            ...     operation(name="add", needs=["a", "b1"], provides=["ab1"])(add),
+            ...     operation(name="sub", needs=["a", optional("b2")], provides=["ab2"])(lambda a, b=1: a-b),
+            ...     operation(name="abb", needs=["ab1", "ab2"], provides=["asked"])(add),
+            ... )
 
-        >>> netop.plot(True);                       # plot just the graph in a matplotlib window # doctest: +SKIP
-        >>> inputs = {'a': 1, 'b1': 2}
-        >>> solution = netop(**inputs)              # now plots will include the execution-plan
+            >>> pipeline.plot(True);                       # plot just the graph in a matplotlib window # doctest: +SKIP
+            >>> inputs = {'a': 1, 'b1': 2}
+            >>> solution = pipeline(**inputs)              # now plots will include the execution-plan
 
-        >>> netop.plot('plot1.svg', inputs=inputs, outputs=['asked', 'b1'], solution=solution);           # doctest: +SKIP
-        >>> dot = netop.plot(solution=solution);    # just get the `pydot.Dot` object, renderable in Jupyter
-        >>> print(dot)
-        digraph netop {
-        fontname=italic;
-        label=<netop>;
-        <a> [fillcolor=wheat, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
-        ...
+        The solution is also *plottable*:
+
+            >>> solution.plot('plot1.svg');                                                             # doctest: +SKIP
+
+        or you may augment the pipelinewith the requested inputs/outputs & solution:
+
+            >>> pipeline.plot('plot1.svg', inputs=inputs, outputs=['asked', 'b1'], solution=solution);  # doctest: +SKIP
+
+        In any case you may get the `pydot.Dot` object
+        (n.b. it is renderable in Jupyter as-is):
+
+            >>> dot = pipeline.plot(solution=solution);
+            >>> print(dot)
+            digraph pipeline {
+            fontname=italic;
+            label=<pipeline>;
+            <a> [fillcolor=wheat, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
+            ...
 
         .. graphtik::
 
@@ -697,13 +715,13 @@ class Plottable(abc.ABC):
         You may use the :attr:`PlotArgs.graph` overlay to skip certain nodes (or edges)
         from the plots:
 
-        >>> import networkx as nx
+            >>> import networkx as nx
 
-        >>> g = nx.DiGraph()  # the overlay
-        >>> to_hide = netop.net.find_op_by_name("sub")
-        >>> g.add_node(to_hide, no_plot=True)
-        >>> dot = netop.plot(graph=g)
-        >>> assert "<sub>" not in str(dot), str(dot)
+            >>> g = nx.DiGraph()  # the overlay
+            >>> to_hide = pipeline.net.find_op_by_name("sub")
+            >>> g.add_node(to_hide, no_plot=True)
+            >>> dot = pipeline.plot(graph=g)
+            >>> assert "<sub>" not in str(dot), str(dot)
 
         .. graphtik::
 
@@ -725,6 +743,7 @@ class Plottable(abc.ABC):
         plot_args = PlotArgs(**kw)
 
         from .plot import Plotter, Theme, get_active_plotter
+        from .execution import Solution
 
         ## Ensure a valid plotter in the args asap.
         #
@@ -733,6 +752,7 @@ class Plottable(abc.ABC):
         if not plotter:
             plotter = get_active_plotter()
         plot_args = plot_args._replace(plotter=plotter)
+        assert plot_args.plotter, f"Expected `plotter`: {plot_args}"
 
         ## Overwrite any dictionaries over active theme asap.
         #
@@ -741,7 +761,14 @@ class Plottable(abc.ABC):
             plot_args = plot_args._replace(theme=theme)
 
         plot_args = self.prepare_plot_args(plot_args)
-        assert plot_args.graph, plot_args
+        assert plot_args.graph, f"Expected `graph: {plot_args}"
+
+        plot_args = plot_args.with_defaults(
+            # Don't leave `solution` unassigned, if possible.
+            solution=plot_args.plottable
+            if isinstance(plot_args.plottable, Solution)
+            else None
+        )
 
         return plot_args.plotter.plot(plot_args)
 
@@ -753,7 +780,132 @@ class Plottable(abc.ABC):
         - Clone the graph or merge it with the one in the `plot_args`
           (see :meth:`PlotArgs.clone_or_merge_graph()`.
 
-        - For the rest args, prefer :meth:`PlotArgs.with_defaults()` over :meth:`._replace()`,
+        - For the rest args, prefer :meth:`PlotArgs.with_defaults()` over
+          :meth:`_replace()`,
           not to override user args.
         """
-        pass
+
+
+class RenArgs(NamedTuple):
+    """
+    Arguments received by callbacks in :meth:`.rename()` and :term:`operation nesting`.
+    """
+
+    #: what is currently being renamed,
+    #: one of the string: ``(op | needs | provides | aliases)``
+    typ: str
+    #: the operation currently being processed
+    op: "Operation"
+    # the name of the item to be renamed/nested
+    name: str
+    #: The parent :class:`.Pipeline` of the operation currently being processed,.
+    #: Has value only when doing :term:`operation nesting` from :func:`.compose()`.
+    parent: "Pipeline" = None
+
+
+class Operation(Plottable, abc.ABC):
+    """An abstract class representing an action with :meth:`.compute()`."""
+
+    name: str
+    needs: Items
+    op_needs: Items
+    provides: Items
+    op_provides: Items
+
+    @property
+    def __name__(self) -> str:
+        return self.name
+
+    @abc.abstractmethod
+    def compute(self, named_inputs, outputs=None):
+        """
+        Compute (optional) asked `outputs` for the given `named_inputs`.
+
+        It is called by :class:`.Network`.
+        End-users should simply call the operation with `named_inputs` as kwargs.
+
+        :param named_inputs:
+            the input values with which to feed the computation.
+        :returns list:
+            Should return a list values representing
+            the results of running the feed-forward computation on
+            ``inputs``.
+        """
+
+    # def plot(self, *args, **kw):
+    #     """Dead impl so as to be easier to make a dummy Op."""
+    #     raise NotImplementedError("Operation subclasses")
+
+    def _rename_graph_names(
+        self, kw, renamer: Union[Callable[[RenArgs], str], Mapping[str, str]],
+    ) -> None:
+        """
+        Pass operation & dependency names through `renamer`.
+
+        :param kw:
+            all data are extracted 1st from this kw, falling back on the operation's
+            attributes, and it is modified in-place
+
+        For the other 2 params, see :meth:`.FunctionalOperation.withset()`.
+
+        :raise ValueError:
+            - if a `renamer` was neither dict nor callable
+            - if a `renamer` dict contained a non-string value,
+        """
+
+        def rename_driver(ren_args: RenArgs) -> str:
+            """Handle dicts, callables and non-string names as true/false."""
+            from .modifiers import dep_renamed
+
+            ok = False
+            try:
+                new_name = old_name = ren_args.name
+                if isinstance(renamer, cabc.Mapping):
+                    dst = renamer.get(old_name)
+                    if callable(dst) or (dst and isinstance(dst, str)):
+                        new_name = dep_renamed(old_name, dst)
+                elif callable(renamer):
+                    dst = renamer(ren_args)
+                    if dst and isinstance(dst, str):
+                        new_name = dst
+                    # A falsy means don't touch the node.
+                else:
+                    raise AssertionError(
+                        f"Invalid `renamer` {renamer!r} should have been caught earlier."
+                    )
+
+                if not new_name or not isinstance(new_name, str):
+                    raise ValueError(
+                        f"Must rename {old_name!r} into a non-empty string, got {new_name!r}!"
+                    )
+
+                ok = True
+                return new_name
+            finally:
+                if not ok:
+                    # Debug aid without touching ex.
+                    log.warning("Failed to rename %s", ren_args)
+
+        ren_args = RenArgs(None, self, None)
+
+        kw["name"] = rename_driver(
+            ren_args._replace(typ="op", name=kw.get("name", self.name))
+        )
+        ren_args = ren_args._replace(typ="needs")
+        kw["needs"] = [
+            rename_driver(ren_args._replace(name=n))
+            for n in kw.get("needs", self.needs)
+        ]
+        ren_args = ren_args._replace(typ="provides")
+        # Store renamed `provides` as map, used for `aliases` below.
+        renamed_provides = {
+            n: rename_driver(ren_args._replace(name=n))
+            for n in kw.get("provides", self.provides)
+        }
+        kw["provides"] = list(renamed_provides.values())
+        if hasattr(self, "aliases"):
+            ren_args = ren_args._replace(typ="aliases")
+            kw["aliases"] = [
+                (renamed_provides[k], rename_driver(ren_args._replace(name=v)),)
+                for k, v in kw.get("aliases", self.aliases)  # pylint: disable=no-member
+            ]
