@@ -17,14 +17,12 @@ from .base import (
     AbortedException,
     IncompleteExecutionError,
     Items,
-    PlotArgs,
-    Plottable,
     aslist,
     astuple,
+    first_solid,
     jetsam,
 )
 from .config import (
-    first_solid,
     get_execution_pool,
     is_abort,
     is_debug,
@@ -41,7 +39,7 @@ from .network import (
     yield_node_names,
     yield_ops,
 )
-from .composition import Operation
+from .base import Operation, PlotArgs, Plottable
 
 #: If this logger is *eventually* DEBUG-enabled,
 #: the string-representation of network-objects (network, plan, solution)
@@ -55,7 +53,7 @@ def _isDebugLogging():
 
 class Solution(ChainMap, Plottable):
     """
-    A chain-map collecting :term:`solution` outputs and execution state (eg :term:`overwrites`)
+    The :term:`solution` chain-map and execution state (e.g. :term:`overwrite` or :term:`canceled operation`)
     """
 
     def __init__(self, plan, input_values: dict):
@@ -239,7 +237,7 @@ class Solution(ChainMap, Plottable):
         return {k: v for k, v in dd.items() if len(v) > 1}
 
     def check_if_incomplete(self) -> Optional[IncompleteExecutionError]:
-        """Return a :class:`IncompleteExecutionError` if `netop` operations failed/canceled. """
+        """Return a :class:`IncompleteExecutionError` if `pipeline` operations failed/canceled. """
         failures = {
             op: ex for op, ex in self.executed.items() if isinstance(ex, Exception)
         }
@@ -263,7 +261,7 @@ class Solution(ChainMap, Plottable):
             return IncompleteExecutionError(msg, self)
 
     def scream_if_incomplete(self):
-        """Raise a :class:`IncompleteExecutionError` if `netop` operations failed/canceled. """
+        """Raise a :class:`IncompleteExecutionError` if `pipeline` operations failed/canceled. """
         ex = self.check_if_incomplete()
         if ex:
             raise ex
@@ -714,7 +712,7 @@ class ExecutionPlan(
             )
 
             log.debug(
-                "=== (%s) Executing netop(%s)%s%s, on inputs%s, according to %s...",
+                "=== (%s) Executing pipeline(%s)%s%s, on inputs%s, according to %s...",
                 solution.solid,
                 name,
                 ", in parallel" if in_parallel else "",
@@ -733,7 +731,7 @@ class ExecutionPlan(
                 if _isDebugLogging():
                     elapsed = sum(solution.elapsed_ms.values())
                     log.debug(
-                        "=== (%s) Completed netop(%s) in %0.3fms.",
+                        "=== (%s) Completed pipeline(%s) in %0.3fms.",
                         solution.solid,
                         name,
                         elapsed,
