@@ -21,10 +21,10 @@ from typing import (
 import networkx as nx
 from boltons.setutils import IndexedSet as iset
 
-from .base import Items, PlotArgs, Plottable, astuple, jetsam
+from .base import Items, astuple, jetsam
 from .config import is_debug, is_skip_evictions
 from .modifiers import dep_renamed, is_mapped, is_optional, is_sfx, optional
-from .composition import FunctionalOperation, Operation
+from .base import Operation, PlotArgs, Plottable, RenArgs
 
 NodePredicate = Callable[[Any, Mapping], bool]
 
@@ -609,28 +609,14 @@ class Network(Plottable):
         return plan
 
 
-class NestArgs(NamedTuple):
-    """:term:`operation nesting` callables receive instances of this class."""
-
-    #: the parent :class:`.NetworkOperation` of the operation currently being processed
-    parent: "NetworkOperation"
-    #: what is currently being renamed,
-    #: one of the string: ``(op | needs | provides)``
-    typ: str
-    #: the operation currently being processed
-    op: Operation
-    # the name of the item to be renamed/nested
-    name: str
-
-
 def build_network(
     operations,
     rescheduled=None,
     endured=None,
     parallel=None,
     marshalled=None,
-    nest=None,
     node_props=None,
+    renamer=None,
 ):
     """
     The :term:`network` factory that does :term:`operation merging` before constructing it.
@@ -638,87 +624,38 @@ def build_network(
     :param nest:
         see same-named param in :func:`.compose`
     """
-    from boltons.setutils import IndexedSet as iset
-    from .composition import NULL_OP, NetworkOperation
+    kw = {
+        k: v for k, v in locals().items() if v is not None and k not in ("operations")
+    }
+
+    from .pipeline import NULL_OP, Pipeline
 
     def proc_op(op, parent=None):
         """clone FuncOperation with certain props changed"""
-        assert isinstance(op, FunctionalOperation), op
+        from .op import FunctionalOperation
 
-        ## Convey any node-props specified in the netop here
+        ## Convey any node-props specified in the pipeline here
         #  to all sub-operations.
         #
-        if (
-            node_props
-            or nest
-            or rescheduled is not None
-            or endured is not None
-            or parallel is not None
-            or marshalled is not None
-        ):
-            kw = {
-                k: v
-                for k, v in [
-                    ("rescheduled", rescheduled),
-                    ("endured", endured),
-                    ("parallel", parallel),
-                    ("marshalled", marshalled),
-                ]
-                if v is not None
-            }
+        if kw and isinstance(op, FunctionalOperation):
             if node_props:
-                op_node_props = op.node_props.copy()
-                op_node_props.update(node_props)
-                kw["node_props"] = op_node_props
+                kw["node_props"] = {**op.node_props, **node_props}
 
-            ## If `nest`, rename the op & data (predicated by `nest`)
-            #  by prefixing them with their parent netop.
-            #
-            nest_args = NestArgs(parent, op, None, None)
-            if nest:
-                kw["name"] = rename(nest_args._replace(typ="op", name=op.name))
-                kw["needs"] = [
-                    rename(nest_args._replace(typ="needs", name=n)) for n in op.needs
-                ]
-                kw["provides"] = [
-                    rename(nest_args._replace(typ="provides", name=n))
-                    for n in op.provides
-                ]
+            if callable(renamer):
 
+                def parent_wrapper(ren_args: RenArgs) -> str:
+                    # Provide RenArgs.parent.
+                    return renamer(ren_args._replace(parent=parent))
+
+                kw["renamer"] = parent_wrapper
             op = op.withset(**kw)
 
         return op
 
-    def rename(nest_args: NestArgs) -> str:
-        """Handle user's or default `nest` callable's results."""
-        assert callable(nest), (nest, nest_args)
-
-        ok = False
-        try:
-            ret = nest(nest_args)
-            if not ret:
-                # A falsy means don't touch the node.
-                ret = nest_args.name
-            elif not isinstance(ret, str):
-                # Truthy but not str values mean apply default nesting.
-                ret = nest_any_node(nest_args)
-
-            ok = True
-            return ret
-        finally:
-            if not ok:
-                log.warning("Failed to nest-rename %s", nest_args)
-
-    if nest:
-        ## Set default nesting if not one provided by user.
-        #
-        if not callable(nest):
-            nest = nest_any_node
-
     merge_set = iset()  # Preseve given node order.
     for op in operations:
-        if isinstance(op, NetworkOperation):
-            merge_set.update(proc_op(s, op) for s in yield_ops(op.net.graph))
+        if isinstance(op, Pipeline):
+            merge_set.update(proc_op(s, op) for s in op.ops)
         else:
             merge_set.add(proc_op(op))
     merge_set = iset(i for i in merge_set if not isinstance(i, NULL_OP))
@@ -727,20 +664,3 @@ def build_network(
     net = Network(*merge_set)
 
     return net
-
-
-def nest_any_node(nest_args: NestArgs) -> str:
-    """Nest both operation & data under `parent`'s name (if given).
-
-    :return:
-        the nested name of the operation or data
-    """
-
-    def prefixed(name):
-        return f"{nest_args.parent.name}.{name}" if nest_args.parent else name
-
-    return (
-        prefixed(nest_args.name)
-        if nest_args.typ == "op"
-        else dep_renamed(nest_args.name, prefixed)
-    )
