@@ -30,3 +30,6 @@ def pytest_sessionstart(session):
 
 def pytest_configure(config):
     config.addinivalue_line("markers", "sphinx: parametrized sphinx test-launches")
+    config.addinivalue_line(
+        "markers", "test_params: for parametrized sphinx test-launches"
+    )
