@@ -15,31 +15,30 @@ from multiprocessing import dummy as mp_dummy
 from multiprocessing import get_context
 from operator import add, floordiv, mul, sub
 from pprint import pprint
+from textwrap import dedent
 from typing import Tuple
 from unittest.mock import MagicMock
 
 import pytest
 
-from graphtik import (
-    NO_RESULT,
-    NULL_OP,
-    AbortedException,
-    IncompleteExecutionError,
+from graphtik import network
+from graphtik.base import AbortedException, IncompleteExecutionError
+from graphtik.config import (
     abort_run,
-    compose,
+    debug_enabled,
     evictions_skipped,
     execution_pool_plugged,
     get_execution_pool,
     is_marshal_tasks,
-    network,
-    operation,
     operations_endured,
     operations_reschedullled,
     tasks_marshalled,
 )
 from graphtik.execution import Solution
-from graphtik.modifiers import optional, sfx, sfxed, vararg
-from graphtik.composition import NetworkOperation, Operation
+from graphtik.base import Operation
+from graphtik.modifiers import dep_renamed, optional, sfx, sfxed, vararg
+from graphtik.op import NO_RESULT, NO_RESULT_BUT_SFX, operation
+from graphtik.pipeline import NULL_OP, Pipeline, compose
 
 log = logging.getLogger(__name__)
 
@@ -171,7 +170,7 @@ def test_smoke_test():
     )
 
     # compose network
-    netop = compose("my network", sum_op1, mul_op1, pow_op1, sum_op2, sum_op3)
+    pipeline = compose("my network", sum_op1, mul_op1, pow_op1, sum_op2, sum_op3)
 
     #
     # Running the network
@@ -189,20 +188,20 @@ def test_smoke_test():
         "sum_ab_p3": 27.0,
         "sum_ab_times_b": 6,
     }
-    assert netop(a=1, b=2) == exp
+    assert pipeline(a=1, b=2) == exp
 
     # get specific outputs
     exp = {"sum_ab_times_b": 6}
-    assert netop.compute({"a": 1, "b": 2}, ["sum_ab_times_b"]) == exp
+    assert pipeline.compute({"a": 1, "b": 2}, ["sum_ab_times_b"]) == exp
 
     # start with inputs already computed
     exp = {"sum_ab_times_b": 2}
-    assert netop.compute({"sum_ab": 1, "b": 2}, ["sum_ab_times_b"]) == exp
+    assert pipeline.compute({"sum_ab": 1, "b": 2}, ["sum_ab_times_b"]) == exp
 
     with pytest.raises(ValueError, match="Unknown output node"):
-        netop.compute({"sum_ab": 1, "b": 2}, "bad_node")
+        pipeline.compute({"sum_ab": 1, "b": 2}, "bad_node")
     with pytest.raises(ValueError, match="Unknown output node"):
-        netop.compute({"sum_ab": 1, "b": 2}, ["b", "bad_node"])
+        pipeline.compute({"sum_ab": 1, "b": 2}, ["b", "bad_node"])
 
 
 def test_network_plan_execute():
@@ -279,8 +278,53 @@ def test_network_plan_execute():
     assert sol == exp
 
 
-def test_network_nest_errors(caplog):
-    def screamy_nester(nest_args):
+def test_compose_rename_dict(caplog):
+    pip = compose(
+        "t",
+        operation(str, "op1", provides=["a", "aa"]),
+        operation(
+            str,
+            "op2",
+            needs="a",
+            provides=["b", sfx("c")],
+            aliases=[("b", "B"), ("b", "p")],
+        ),
+        nest={"op1": "OP1", "op2": lambda n: "OP2", "a": "A", "b": "bb"},
+    )
+    print(str(pip))
+    assert str(pip) == (
+        "Pipeline('t', needs=['A'], "
+        "provides=['A', 'aa', 'bb', sfx('c'), 'B', 'p'], x2 ops: OP1, OP2)"
+    )
+    print(str(pip.ops))
+    assert (
+        str(pip.ops)
+        == dedent(
+            """
+        [FunctionalOperation(name='OP1', provides=['A', 'aa'], fn='str'),
+         FunctionalOperation(name='OP2', needs=['A'], provides=['bb', sfx('c')],
+         aliases=[('bb', 'B'), ('bb', 'p')], fn='str')]
+    """
+        ).replace("\n", "")
+    )
+
+
+def test_compose_rename_dict_non_str(caplog):
+    pip = compose("t", operation(str, "op1"), operation(str, "op2"), nest={"op1": 1},)
+    exp = "Pipeline('t', x2 ops: op1, op2)"
+    print(pip)
+    assert str(pip) == exp
+    exp = "Pipeline('t', x2 ops: t.op1, op2)"
+    pip = compose("t", pip, nest={"op1": 1, "op2": 0})
+    assert str(pip) == exp
+    pip = compose("t", pip, nest={"op1": 1, "op2": ""})
+    assert str(pip) == exp
+    for record in caplog.records:
+        assert "Failed to nest-rename" not in record.message
+
+
+def test_compose_rename_bad_screamy(caplog):
+    def screamy_nester(ren_args):
         raise RuntimeError("Bluff")
 
     with pytest.raises(RuntimeError, match="Bluff"):
@@ -292,12 +336,22 @@ def test_network_nest_errors(caplog):
         )
     for record in caplog.records:
         if record.levelname == "WARNING":
-            assert "(parent=None, typ='op', op=None, name='op1')" in record.message
+            assert "name='op1', parent=None)" in record.message
 
 
-def test_network_nest_ops_only():
-    def ops_only(nest_args):
-        return nest_args.typ == "op"
+def test_compose_rename_preserve_ops(caplog):
+    pip = compose(
+        "t",
+        operation(str, "op1"),
+        operation(str, "op2"),
+        nest=lambda na: f"aa.{na.name}",
+    )
+    assert str(pip) == "Pipeline('t', x2 ops: aa.op1, aa.op2)"
+
+
+def test_compose_merge_ops():
+    def ops_only(ren_args):
+        return ren_args.typ == "op"
 
     sum_op1 = operation(name="sum_op1", needs=["a", "b"], provides="sum1")(add)
     sum_op2 = operation(name="sum_op2", needs=["a", "b"], provides="sum2")(add)
@@ -332,7 +386,7 @@ def test_network_nest_ops_only():
     assert sol == exp
 
     assert repr(net3).startswith(
-        "NetworkOperation('merged', needs=['a', 'b', 'sum1', 'c', 'd', 'e', 'f'], "
+        "Pipeline('merged', needs=['a', 'b', 'sum1', 'c', 'd', 'e', 'f'], "
         "provides=['sum1', 'sum2', 'sum3', 'a', 'b'], x5 ops"
     )
 
@@ -349,7 +403,7 @@ def test_network_combine():
     exp = {"a": 1, "b": 2, "c": 4, "sum1": 3, "sum2": 3, "sum3": 7}
     assert net1(a=1, b=2, c=4) == exp
     assert repr(net1).startswith(
-        "NetworkOperation('my network 1', needs=['a'(?), 'b', 'sum1', 'c'], "
+        "Pipeline('my network 1', needs=['a'(?), 'b', 'sum1', 'c'], "
         "provides=['sum1', 'sum2', 'sum3'], x3 ops"
     )
 
@@ -361,7 +415,7 @@ def test_network_combine():
     exp = {"a": 1, "b": 2, "sum1": 3, "sum2": 5}
     assert net2(**{"a": 1, "b": 2}) == exp
     assert repr(net2).startswith(
-        "NetworkOperation('my network 2', needs=['a'(?), 'b', 'sum1'], provides=['sum1', 'sum2'], x2 ops"
+        "Pipeline('my network 2', needs=['a'(?), 'b', 'sum1'], provides=['sum1', 'sum2'], x2 ops"
     )
 
     net3 = compose("merged", net1, net2)
@@ -369,7 +423,7 @@ def test_network_combine():
     assert net3(a=1, b=2, c=4) == exp
 
     assert repr(net3).startswith(
-        "NetworkOperation('merged', needs=['a'(?), 'b', 'sum1', 'c'], provides=['sum1', 'sum2', 'sum3'], x4 ops"
+        "Pipeline('merged', needs=['a'(?), 'b', 'sum1', 'c'], provides=['sum1', 'sum2', 'sum3'], x4 ops"
     )
 
     ## Reverse ops, change results and `needs` optionality.
@@ -379,7 +433,7 @@ def test_network_combine():
     assert net3(**{"a": 1, "b": 2, "c": 4}) == exp
 
     assert repr(net3).startswith(
-        "NetworkOperation('merged', needs=['a'(?), 'b', 'sum1', 'c'], provides=['sum1', 'sum2', 'sum3'], x4 ops"
+        "Pipeline('merged', needs=['a'(?), 'b', 'sum1', 'c'], provides=['sum1', 'sum2', 'sum3'], x4 ops"
     )
 
 
@@ -412,8 +466,8 @@ def test_network_merge_in_doctests():
     week = compose("week", *weekdays, nest=True)
     assert len(week.ops) == 6
 
-    def nester(nest_args):
-        if nest_args.name not in ("backlog", "tasks done", "todos"):
+    def nester(ren_args):
+        if ren_args.name not in ("backlog", "tasks done", "todos"):
             return True
 
     week = compose("week", *weekdays, nest=nester)
@@ -428,8 +482,55 @@ def test_network_merge_in_doctests():
         "day 2.tasks": "a lot!",
     }
 
-    dot = sol.plot()
-    assert len(dot.get_subgraphs()) == days_count
+
+def test_compose_nest_dict(caplog):
+    pipe = compose(
+        "t",
+        compose(
+            "p1",
+            operation(
+                str,
+                name="op1",
+                needs=[sfx("a"), "aa"],
+                provides=[sfxed("S1", "g"), sfxed("S2", "h")],
+            ),
+        ),
+        compose(
+            "p2",
+            operation(
+                str,
+                name="op2",
+                needs=sfx("a"),
+                provides=["a", sfx("b")],
+                aliases=[("a", "b")],
+            ),
+        ),
+        nest={
+            "op1": True,
+            "op2": lambda n: "p2.op2",
+            "aa": False,
+            sfx("a"): True,
+            "b": lambda n: f"PP.{n}",
+            sfxed("S1", "g"): True,
+            sfxed("S2", "h"): lambda n: dep_renamed(n, "ss2"),
+            sfx("b"): True,
+        },
+    )
+    got = str(pipe.ops)
+    print(got)
+    assert got == re.sub(
+        r"[\n ]{2,}",  # collapse all space-chars into a single space
+        " ",
+        """
+        [FunctionalOperation(name='p1.op1', needs=[sfx('p1.a'), 'aa'],
+         provides=[sfxed('p1.S1', 'g'), sfxed('ss2', 'h')], fn='str'),
+        FunctionalOperation(name='p2.op2', needs=[sfx('p2.a')],
+         provides=['a', sfx('p2.b')], aliases=[('a', 'PP.b')], fn='str')]
+
+        """.strip(),
+    )
+    for record in caplog.records:
+        assert record.levelname != "WARNING"
 
 
 def test_aliases(exemethod):
@@ -457,7 +558,7 @@ def samplenet():
 
 
 def test_node_predicate_based_prune():
-    netop = compose(
+    pipeline = compose(
         "N",
         operation(name="A", needs=["a"], provides=["aa"], node_props={"color": "red"})(
             identity
@@ -473,16 +574,16 @@ def test_node_predicate_based_prune():
         )(addall),
     )
     inp = {"a": 1, "b": 2, "c": 3}
-    assert netop(**inp)["sum"] == 6
-    assert len(netop.net.graph.nodes) == 11
+    assert pipeline(**inp)["sum"] == 6
+    assert len(pipeline.net.graph.nodes) == 11
 
     pred = lambda n, d: d.get("color", None) != "red"
-    assert netop.withset(predicate=pred)(**inp)["sum"] == 5
-    assert len(netop.withset(predicate=pred).compile().dag.nodes) == 9
+    assert pipeline.withset(predicate=pred)(**inp)["sum"] == 5
+    assert len(pipeline.withset(predicate=pred).compile().dag.nodes) == 9
 
     pred = lambda n, d: "color" not in d
-    assert netop.withset(predicate=pred)(**inp)["sum"] == 3
-    assert len(netop.withset(predicate=pred).compile().dag.nodes) == 7
+    assert pipeline.withset(predicate=pred)(**inp)["sum"] == 3
+    assert len(pipeline.withset(predicate=pred).compile().dag.nodes) == 7
 
 
 def test_input_based_pruning():
@@ -555,17 +656,17 @@ def test_pruning_raises_for_bad_output(samplenet):
 
 
 def test_impossible_outputs():
-    netop = compose(
+    pipeline = compose(
         "test_net",
         operation(name="op1", needs=["a"], provides="aa")(identity),
         operation(name="op2", needs=["aa", "bb"], provides="aabb")(identity),
     )
     with pytest.raises(ValueError) as exinfo:
-        netop.compute({"a": 1,}, ["aabb"])
+        pipeline.compute({"a": 1,}, ["aabb"])
     assert exinfo.match("Unreachable outputs")
 
     with pytest.raises(ValueError) as exinfo:
-        netop.compute({"a": 1,}, ["aa", "aabb"])
+        pipeline.compute({"a": 1,}, ["aa", "aabb"])
     assert exinfo.match("Unreachable outputs")
 
 
@@ -713,10 +814,11 @@ def test_same_outputs_operations_order():
     assert addsub(**inp) == {"a": 3, "b": 1, "ab": 2}
     assert addsub.compute(inp, "ab") == {"ab": 2}
     assert subadd(**inp) == {"a": 3, "b": 1, "ab": 4}
-    assert subadd.compute(inp, "ab") == {"ab": 4}
+    sol = subadd.compute(inp, "ab")
+    assert sol == {"ab": 4}
 
     # ## Check it does not duplicate evictions
-    assert len(subadd.last_plan.steps) == 4
+    assert len(sol.plan.steps) == 4
 
     ## Add another step to test evictions
     #
@@ -739,8 +841,9 @@ def test_same_outputs_operations_order():
     assert solution == {"AB": 4}
     assert solution.overwrites == {}
 
-    assert subadd.compute(inp, "AB") == {"AB": 4}
-    assert len(subadd.last_plan.steps) == 6
+    sol = subadd.compute(inp, "AB")
+    assert sol == {"AB": 4}
+    assert len(sol.plan.steps) == 6
 
 
 def test_same_inputs_evictions():
@@ -753,9 +856,10 @@ def test_same_inputs_evictions():
 
     inp = {"a": 3}
     assert pipeline(**inp) == {"a": 3, "2a": 6, "@S": 6}
-    assert pipeline.compute(inp, "@S") == {"@S": 6}
+    sol = pipeline.compute(inp, "@S")
+    assert sol == {"@S": 6}
     ## Check it does not duplicate evictions
-    assert len(pipeline.last_plan.steps) == 4
+    assert len(sol.plan.steps) == 4
 
 
 def test_unsatisfied_operations(exemethod):
@@ -841,56 +945,54 @@ def test_narrow_and_optionality(reverse):
     if reverse:
         ops = list(reversed(ops))
         provides = "'sum2', 'sum1'"
-    netop_str = (
-        f"NetworkOperation('t', needs=['a', 'bb'(?)], provides=[{provides}], x2 ops"
-    )
+    pipeline_str = f"Pipeline('t', needs=['a', 'bb'(?)], provides=[{provides}], x2 ops"
 
-    netop = compose("t", *ops)
-    assert repr(netop).startswith(netop_str)
+    pipeline = compose("t", *ops)
+    assert repr(pipeline).startswith(pipeline_str)
 
     ## IO & predicate do not affect network, but solution.
 
     ## Compose with `inputs`
     #
-    netop = compose("t", *ops)
-    assert repr(netop).startswith(netop_str)
-    assert repr(netop.compile("a")).startswith(
+    pipeline = compose("t", *ops)
+    assert repr(pipeline).startswith(pipeline_str)
+    assert repr(pipeline.compile("a")).startswith(
         f"ExecutionPlan(needs=['a'], provides=[{provides}], x2 steps:"
     )
     #
-    netop = compose("t", *ops)
-    assert repr(netop).startswith(netop_str)
-    assert repr(netop.compile(["bb"])).startswith(
+    pipeline = compose("t", *ops)
+    assert repr(pipeline).startswith(pipeline_str)
+    assert repr(pipeline.compile(["bb"])).startswith(
         "ExecutionPlan(needs=['bb'(?)], provides=['sum1'], x1 steps:"
     )
 
     ## Narrow by `provides`
     #
-    netop = compose("t", *ops, outputs="sum1")
-    assert repr(netop).startswith(netop_str)
-    assert repr(netop.compile("bb")).startswith(
+    pipeline = compose("t", *ops, outputs="sum1")
+    assert repr(pipeline).startswith(pipeline_str)
+    assert repr(pipeline.compile("bb")).startswith(
         "ExecutionPlan(needs=['bb'(?)], provides=['sum1'], x3 steps:"
     )
-    assert repr(netop.compile("bb")) == repr(netop.compute({"bb": 1}).plan)
+    assert repr(pipeline.compile("bb")) == repr(pipeline.compute({"bb": 1}).plan)
 
-    netop = compose("t", *ops, outputs=["sum2"])
-    assert repr(netop).startswith(netop_str)
-    assert not netop.compile("bb").steps
-    assert len(netop.compile("a").steps) == 3
-    assert repr(netop.compile("a")).startswith(
+    pipeline = compose("t", *ops, outputs=["sum2"])
+    assert repr(pipeline).startswith(pipeline_str)
+    assert not pipeline.compile("bb").steps
+    assert len(pipeline.compile("a").steps) == 3
+    assert repr(pipeline.compile("a")).startswith(
         "ExecutionPlan(needs=['a'], provides=['sum2'], x3 steps:"
     )
 
     ## Narrow by BOTH
     #
-    netop = compose("t", *ops, outputs=["sum1"])
-    assert repr(netop.compile(inputs="a")).startswith(
+    pipeline = compose("t", *ops, outputs=["sum1"])
+    assert repr(pipeline.compile(inputs="a")).startswith(
         "ExecutionPlan(needs=['a'(?)], provides=['sum1'], x3 steps:"
     )
 
-    netop = compose("t", *ops, outputs=["sum2"])
+    pipeline = compose("t", *ops, outputs=["sum2"])
     with pytest.raises(ValueError, match="Unsolvable graph:"):
-        netop.compute({"bb": 11})
+        pipeline.compute({"bb": 11})
 
 
 # Function without return value.
@@ -904,7 +1006,7 @@ def _box_increment(box):
 
 
 @pytest.fixture(params=[0, 1])
-def netop_sideffect1(request, exemethod) -> NetworkOperation:
+def pipeline_sideffect1(request, exemethod) -> Pipeline:
     ops = [
         operation(name="extend", needs=["box", sfx("a")], provides=[sfx("b")])(
             _box_extend
@@ -921,12 +1023,12 @@ def netop_sideffect1(request, exemethod) -> NetworkOperation:
     return graph
 
 
-def test_sideffect_no_real_data(netop_sideffect1: NetworkOperation):
+def test_sideffect_no_real_data(pipeline_sideffect1: Pipeline):
     sidefx_fail = is_marshal_tasks() and not isinstance(
         get_execution_pool(), types.FunctionType  # mp_dummy.Pool
     )
 
-    graph = netop_sideffect1
+    graph = pipeline_sideffect1
     inp = {"box": [0], "a": True}
 
     ## Normal data must not match sideffects.
@@ -1000,20 +1102,20 @@ def test_sideffect_real_input(reverse, exemethod):
     }
 
 
-def test_sideffect_steps(exemethod, netop_sideffect1: NetworkOperation):
+def test_sideffect_steps(exemethod, pipeline_sideffect1: Pipeline):
     sidefx_fail = is_marshal_tasks() and not isinstance(
         get_execution_pool(), types.FunctionType  # mp_dummy.Pool
     )
 
-    netop = netop_sideffect1.withset(parallel=exemethod)
+    pipeline = pipeline_sideffect1.withset(parallel=exemethod)
     box_orig = [0]
-    sol = netop.compute({"box": [0], sfx("a"): True}, ["box", sfx("c")])
+    sol = pipeline.compute({"box": [0], sfx("a"): True}, ["box", sfx("c")])
     assert sol == {"box": box_orig if sidefx_fail else [1, 2, 3]}
-    assert len(netop.last_plan.steps) == 4
+    assert len(sol.plan.steps) == 4
 
     ## Check sideffect links plotted as blue
     #  (assumes color used only for this!).
-    dot = netop.net.plot()
+    dot = pipeline.net.plot()
     assert "blue" in str(dot)
 
 
@@ -1023,46 +1125,63 @@ def test_sideffect_NO_RESULT(caplog, exemethod):
     an_sfx = sfx("b")
     op1 = operation(lambda: NO_RESULT, name="do-SFX", provides=an_sfx)
     op2 = operation(lambda: 1, name="ask-SFX", needs=an_sfx, provides="a")
-    netop = compose("t", op1, op2, parallel=exemethod)
-    sol = netop.compute({}, outputs=an_sfx)
+    pipeline = compose("t", op1, op2, parallel=exemethod)
+    sol = pipeline.compute({}, outputs=an_sfx)
     assert op1 in sol.executed
     assert op2 not in sol.executed
     assert sol == {}
-    sol = netop.compute({})
+    sol = pipeline.compute({})
     assert op1 in sol.executed
     assert op2 in sol.executed
     assert sol == {"a": 1}
-    sol = netop.compute({}, outputs="a")
+    sol = pipeline.compute({}, outputs="a")
     assert op1 in sol.executed
     assert op2 in sol.executed
     assert sol == {"a": 1}
 
     # NO_RESULT cancels sideffects of rescheduled ops.
     #
-    netop = compose("t", op1, op2, rescheduled=True, parallel=exemethod)
-    sol = netop.compute({})
+    pipeline = compose("t", op1, op2, rescheduled=True, parallel=exemethod)
+    sol = pipeline.compute({})
     assert op1 in sol.executed
     assert op2 not in sol.executed
     assert sol == {an_sfx: False}
-    sol = netop.compute({}, outputs="a")
+    sol = pipeline.compute({}, outputs="a")
     assert op2 not in sol.executed
     assert op1 in sol.executed
     assert sol == {}  # an_sfx evicted
 
+    # NO_RESULT_BUT_SFX cancels sideffects of rescheduled ops.
+    #
+    op11 = operation(lambda: NO_RESULT_BUT_SFX, name="do-SFX", provides=an_sfx)
+    pipeline = compose("t", op11, op2, rescheduled=True, parallel=exemethod)
+    sol = pipeline.compute({}, outputs=an_sfx)
+    assert op11 in sol.executed
+    assert op2 not in sol.executed
+    assert sol == {}
+    sol = pipeline.compute({})
+    assert op11 in sol.executed
+    assert op2 in sol.executed
+    assert sol == {"a": 1}
+    sol = pipeline.compute({}, outputs="a")
+    assert op11 in sol.executed
+    assert op2 in sol.executed
+    assert sol == {"a": 1}
+
     ## If NO_RESULT were not translated,
     #  a warning of unknown out might have emerged.
     caplog.clear()
-    netop = compose("t", operation(lambda: 1, provides=an_sfx), parallel=exemethod)
-    netop.compute({}, outputs=an_sfx)
+    pipeline = compose("t", operation(lambda: 1, provides=an_sfx), parallel=exemethod)
+    pipeline.compute({}, outputs=an_sfx)
     for record in caplog.records:
         if record.levelname == "WARNING":
             assert "Ignoring result(1) because no `provides`" in record.message
 
     caplog.clear()
-    netop = compose(
+    pipeline = compose(
         "t", operation(lambda: NO_RESULT, provides=an_sfx), parallel=exemethod
     )
-    netop.compute({}, outputs=an_sfx)
+    pipeline.compute({}, outputs=an_sfx)
     for record in caplog.records:
         assert record.levelname != "WARNING"
 
@@ -1077,10 +1196,10 @@ def test_sideffect_cancel_sfx_only_operation(exemethod):
         rescheduled=True,
     )
     op2 = operation(lambda: 1, name="op2", needs=an_sfx, provides="a")
-    netop = compose("t", op1, op2, parallel=exemethod)
-    sol = netop.compute({})
+    pipeline = compose("t", op1, op2, parallel=exemethod)
+    sol = pipeline.compute({})
     assert sol == {an_sfx: False}
-    sol = netop.compute(outputs=an_sfx)
+    sol = pipeline.compute(outputs=an_sfx)
     assert sol == {an_sfx: False}
 
 
@@ -1094,10 +1213,10 @@ def test_sideffect_cancel(exemethod):
         rescheduled=True,
     )
     op2 = operation(lambda: 1, name="op2", needs=an_sfx, provides="b")
-    netop = compose("t", op1, op2, parallel=exemethod)
-    sol = netop.compute()
+    pipeline = compose("t", op1, op2, parallel=exemethod)
+    sol = pipeline.compute()
     assert sol == {"a": 1, an_sfx: False}
-    sol = netop.compute(outputs="a")
+    sol = pipeline.compute(outputs="a")
     assert sol == {"a": 1}  # an_sfx evicted
     ## SFX both pruned & evicted
     #
@@ -1113,10 +1232,10 @@ def test_sideffect_not_canceled_if_not_resched(exemethod):
         lambda: {an_sfx: False}, name="op1", provides=an_sfx, returns_dict=True
     )
     op2 = operation(lambda: 1, name="op2", needs=an_sfx, provides="b")
-    netop = compose("t", op1, op2, parallel=exemethod)
-    # sol = netop.compute()
+    pipeline = compose("t", op1, op2, parallel=exemethod)
+    # sol = pipeline.compute()
     # assert sol == {an_sfx: False, "b": 1}
-    sol = netop.compute(outputs="b")
+    sol = pipeline.compute(outputs="b")
     assert sol == {"b": 1}
 
     # Check also op with some provides
@@ -1129,10 +1248,10 @@ def test_sideffect_not_canceled_if_not_resched(exemethod):
         returns_dict=True,
     )
     op2 = operation(lambda: 1, name="op2", needs=an_sfx, provides="b")
-    netop = compose("t", op1, op2, parallel=exemethod)
-    sol = netop.compute()
+    pipeline = compose("t", op1, op2, parallel=exemethod)
+    sol = pipeline.compute()
     assert sol == {"a": 1, an_sfx: False, "b": 1}
-    sol = netop.compute(outputs="b")
+    sol = pipeline.compute(outputs="b")
     assert sol == {"b": 1}
 
 
@@ -1443,7 +1562,7 @@ def test_execution_endurance(exemethod, endurance, endured):
 def test_rescheduling(exemethod, resched, rescheduled):
     canc = operation(lambda: None, name="canc", needs=["b"], provides="cc")
     op = compose(
-        "netop",
+        "pipeline",
         operation(lambda: [1], name="op1", provides=["a", "b"], rescheduled=1),
         canc,
         operation(
@@ -1484,7 +1603,7 @@ def test_rescheduling(exemethod, resched, rescheduled):
 def test_rescheduling_NO_RESULT(exemethod):
     partial = operation(lambda: NO_RESULT, name="op1", provides=["a"], rescheduled=1)
     canc = operation(lambda: None, name="canc", needs="a", provides="b")
-    op = compose("netop", partial, canc, parallel=exemethod)
+    op = compose("pipeline", partial, canc, parallel=exemethod)
     sol = op()
     assert canc in sol.canceled
     assert partial in sol.executed
@@ -1638,7 +1757,7 @@ def test_combine_networks(exemethod, bools):
             name="sub2", needs=["a_minus_ab", "c"], provides="a_minus_ab_minus_c"
         )(sub),
         parallel=parallel2,
-        nest=lambda nest_args: nest_args.typ == "op",
+        nest=lambda ren_args: ren_args.typ == "op",
     )
     ## Ensure all old-nodes were prefixed.
     #
