@@ -16,11 +16,16 @@ from graphtik.modifiers import (
 )
 
 
+def dilled(i):
+    return dill.loads(dill.dumps(i))
+
+
 def test_dill_modifier():
     s = sfxed("foo", "gg")
-    s == dill.loads(dill.dumps(s))
+    s == dilled(s)
 
 
+## construct in lambdas, not to crash pytest while modifying core _Modifier
 @pytest.mark.parametrize(
     "mod, exp",
     [
@@ -31,8 +36,8 @@ def test_dill_modifier():
         (lambda: optional("b", "bb"), "b"),
         (lambda: vararg("c"), "c"),
         (lambda: varargs("d"), "d"),
-        (lambda: sfx("e"), "sfx: 'e'"),
-        (lambda: sfx("e", optional=1), "sfx: 'e'"),
+        (lambda: sfx("e"), "sfx('e')"),
+        (lambda: sfx("e", optional=1), "sfx('e')"),
         (lambda: sfxed("f", "a", "b"), "sfxed('f', 'a', 'b')"),
         (lambda: sfxed("f", "ff", fn_kwarg="F"), "sfxed('f', 'ff')",),
         (lambda: sfxed("f", "ff", optional=1, fn_kwarg="F"), "sfxed('f', 'ff')",),
@@ -40,11 +45,13 @@ def test_dill_modifier():
     ],
 )
 def test_modifs_str(mod, exp):
-    got = str(mod())
+    got = mod()
     print(got)
-    assert got == exp
+    assert str(got) == exp
+    assert dilled(str(got)) == exp
 
 
+## construct in lambdas, not to crash pytest while modifying core _Modifier
 @pytest.mark.parametrize(
     "mod, exp",
     [
@@ -55,8 +62,8 @@ def test_modifs_str(mod, exp):
         (lambda: optional("b", "bb"), "'b'(?>'bb')"),
         (lambda: vararg("c"), "'c'(*)"),
         (lambda: varargs("d"), "'d'(+)"),
-        (lambda: sfx("e"), "sfx: 'e'"),
-        (lambda: sfx("e", optional=1), "sfx(?): 'e'"),
+        (lambda: sfx("e"), "sfx('e')"),
+        (lambda: sfx("e", optional=1), "sfx('e'(?))"),
         (lambda: sfxed("f", "a", "b"), "sfxed('f', 'a', 'b')"),
         (lambda: sfxed("f", "ff", fn_kwarg="F"), "sfxed('f'(>'F'), 'ff')",),
         (
@@ -69,9 +76,40 @@ def test_modifs_str(mod, exp):
     ],
 )
 def test_modifs_repr(mod, exp):
-    got = repr(mod())
+    got = mod()
+    print(repr(got))
+    assert repr(got) == exp
+    assert dilled(repr(got)) == exp
+
+
+@pytest.mark.parametrize(
+    "mod, exp",
+    [
+        (mapped("b", None), "mapped('b')"),
+        (mapped("b", ""), "mapped('b')"),
+        (mapped("b", "bb"), "mapped('b', 'bb')"),
+        (optional("b"), "optional('b')"),
+        (optional("b", "bb"), "optional('b', 'bb')"),
+        (vararg("c"), "vararg('c')"),
+        (varargs("d"), "varargs('d')"),
+        (sfx("e"), "sfx('e')"),
+        (sfx("e", optional=1), "sfx('e', 1)"),
+        (sfxed("f", "a", "b"), "sfxed('f', 'a', 'b')"),
+        (sfxed("f", "ff", fn_kwarg="F"), "sfxed('f', 'ff', fn_kwarg='F')",),
+        (
+            sfxed("f", "ff", optional=1, fn_kwarg="F"),
+            "sfxed('f', 'ff', fn_kwarg='F', optional=1)",
+        ),
+        (sfxed("f", "ff", optional=1), "sfxed('f', 'ff', optional=1)"),
+        (sfxed_vararg("f", "a"), "sfxed_vararg('f', 'a')"),
+        (sfxed_varargs("f", "a", "b"), "sfxed_varargs('f', 'a', 'b')"),
+    ],
+)
+def test_modifs_cmd(mod, exp):
+    got = mod.cmd
     print(got)
-    assert got == exp
+    assert str(got) == exp
+    assert dilled(str(got)) == exp
 
 
 def test_recreation():
@@ -114,8 +152,8 @@ def test_sideffected_bad(call, exp):
         (optional("b", "bb"), "'p.b'(?>'bb')"),
         (vararg("c"), "'p.c'(*)"),
         (varargs("d"), "'p.d'(+)"),
-        (sfx("e"), "sfx: 'p.e'"),
-        (sfx("e", optional=1), "sfx(?): 'p.e'"),
+        (sfx("e"), "sfx('p.e')"),
+        (sfx("e", optional=1), "sfx('p.e'(?))"),
         (sfxed("f", "a", "b"), "sfxed('p.f', 'a', 'b')",),
         (sfxed("f", "ff", fn_kwarg="F"), "sfxed('p.f'(>'F'), 'ff')",),
         (sfxed("f", "ff", optional=1, fn_kwarg="F"), "sfxed('p.f'(?>'F'), 'ff')",),
@@ -129,6 +167,9 @@ def test_modifs_rename_fn(mod, exp):
     got = repr(dep_renamed(mod, renamer))
     print(got)
     assert got == exp
+    if hasattr(got, "sideffected"):
+        # Check not just(!) `_repr` has changed.
+        assert got.sideffected == renamer(mod.sideffected)
 
 
 @pytest.mark.parametrize(
@@ -140,7 +181,7 @@ def test_modifs_rename_fn(mod, exp):
         (lambda: optional("b", "bb"), "'D'(?>'bb')"),
         (lambda: vararg("c"), "'D'(*)"),
         (lambda: varargs("d"), "'D'(+)"),
-        (lambda: sfx("e"), "sfx: 'D'"),
+        (lambda: sfx("e"), "sfx('D')"),
         (lambda: sfxed("f", "a", "b", optional=1,), "sfxed('D'(?>'f'), 'a', 'b')",),
         (
             lambda: sfxed("f", "a", "b", optional=1, fn_kwarg="F"),
@@ -149,16 +190,17 @@ def test_modifs_rename_fn(mod, exp):
     ],
 )
 def test_modifs_rename_str(mod, exp):
-    got = repr(dep_renamed(mod(), "D"))
-    print(got)
-    assert got == exp
+    got = dep_renamed(mod(), "D")
+    print(repr(got))
+    assert repr(got) == exp
+    assert dilled(repr(got)) == exp
 
 
 @pytest.mark.parametrize(
     "mod, exp",
     [
         (varargs("d"), "'d'(+)"),
-        (sfx("e", optional=1), "sfx(?): 'e'"),
+        (sfx("e", optional=1), "sfx('e'(?))"),
         (sfxed("f", "a", "b"), "'f'"),
         (sfxed("f", "ff", fn_kwarg="F"), "'f'(>'F')",),
         (sfxed("f", "ff", optional=1, fn_kwarg="F"), "'f'(?>'F')",),
