@@ -5,20 +5,31 @@ import logging
 import re
 from collections import OrderedDict, namedtuple
 from functools import partial
+from textwrap import dedent
 from types import SimpleNamespace
 
 import dill
 import pytest
 
-from graphtik import NO_RESULT, compose, operation, optional, sfx, vararg, varargs
+from graphtik import (
+    NO_RESULT,
+    NO_RESULT_BUT_SFX,
+    compose,
+    operation,
+    optional,
+    sfx,
+    vararg,
+    varargs,
+)
 from graphtik.config import (
     operations_endured,
     operations_reschedullled,
     tasks_in_parallel,
     tasks_marshalled,
 )
+from graphtik.modifiers import dep_renamed
 from graphtik.network import yield_ops
-from graphtik.composition import (
+from graphtik.op import (
     FunctionalOperation,
     Operation,
     as_renames,
@@ -53,11 +64,11 @@ def test_repr_smoke(opname, opneeds, opprovides):
 def test_repr_returns_dict():
     assert (
         str(operation(lambda: None, name="", returns_dict=True))
-        == "FunctionalOperation(name='', needs=[], provides=[], fn{}='<lambda>')"
+        == "FunctionalOperation(name='', fn{}='<lambda>')"
     )
     assert (
         str(operation(lambda: None, name="myname"))
-        == "FunctionalOperation(name='myname', needs=[], provides=[], fn='<lambda>')"
+        == "FunctionalOperation(name='myname', fn='<lambda>')"
     )
 
 
@@ -71,25 +82,14 @@ def test_builder_pattern():
     assert not isinstance(opb, FunctionalOperation)
     assert str(opb).startswith("<function FunctionalOperation.withset at")
 
-    assert (
-        str(opb(sum))
-        == "FunctionalOperation(name='sum', needs=[], provides=[], fn='sum')"
-    )
+    assert str(opb(sum)) == "FunctionalOperation(name='sum', fn='sum')"
 
-    assert (
-        str(opb(name="K"))
-        == "FunctionalOperation(name='K', needs=[], provides=[], fn=None)"
-    )
+    assert str(opb(name="K")) == "FunctionalOperation(name='K', fn=None)"
 
     op = opb.withset(needs=["a", "b"])
     assert isinstance(op, FunctionalOperation)
-    assert (
-        str(op)
-        == "FunctionalOperation(name=None, needs=['a', 'b'], provides=[], fn=None)"
-    )
-    with pytest.raises(
-        ValueError, match=f"Operation was not yet provided with a callable `fn`!"
-    ):
+    assert str(op) == "FunctionalOperation(name=None, needs=['a', 'b'], fn=None)"
+    with pytest.raises(ValueError, match=f"Operation must have a callable `fn` and"):
         op.compute({})
 
     op = op.withset(provides="SUM", fn=sum, name=None)
@@ -133,7 +133,7 @@ def test_func_op_validation(opargs, exp):
         with pytest.raises(type(exp), match=str(exp)):
             reparse_operation_data(*opargs)
     else:
-        assert reparse_operation_data(*opargs) == exp
+        assert reparse_operation_data(*opargs)[:-1] == exp
 
 
 @pytest.mark.parametrize(
@@ -276,21 +276,21 @@ def test_op_node_props():
     assert op.node_props == np
 
 
-def _collect_op_props(netop):
+def _collect_op_props(pipe):
     return {
         k.name: v
-        for k, v in netop.net.graph.nodes.data(True)
+        for k, v in pipe.net.graph.nodes.data(True)
         if isinstance(k, Operation)
     }
 
 
-def test_netop_node_props():
+def test_pipeline_node_props():
     op1 = operation(lambda: None, name="a", node_props={"a": 11, "b": 0, "bb": 2})
     op2 = operation(lambda: None, name="b", node_props={"a": 3, "c": 4})
-    netop = compose("n", op1, op2, node_props={"bb": 22, "c": 44})
+    pipeline = compose("n", op1, op2, node_props={"bb": 22, "c": 44})
 
     exp = {"a": {"a": 11, "b": 0, "bb": 22, "c": 44}, "b": {"a": 3, "bb": 22, "c": 44}}
-    node_props = _collect_op_props(netop)
+    node_props = _collect_op_props(pipeline)
     assert node_props == exp
 
     # Check node-prop sideffects are not modified
@@ -299,25 +299,29 @@ def test_netop_node_props():
     assert op2.node_props == {"a": 3, "c": 4}
 
 
-def test_netop_merge_node_props():
+def test_pipeline_merge_node_props():
     op1 = operation(lambda: None, name="a", node_props={"a": 1})
-    netop1 = compose("n1", op1)
+    pipeline1 = compose("n1", op1)
     op2 = operation(lambda: None, name="a", node_props={"a": 11, "b": 0, "bb": 2})
     op3 = operation(lambda: None, name="b", node_props={"a": 3, "c": 4})
-    netop2 = compose("n2", op2, op3)
+    pipeline2 = compose("n2", op2, op3)
 
-    netop = compose("n", netop1, netop2, node_props={"bb": 22, "c": 44}, nest=True)
+    pipeline = compose(
+        "n", pipeline1, pipeline2, node_props={"bb": 22, "c": 44}, nest=False
+    )
+    exp = {"a": {"a": 1, "bb": 22, "c": 44}, "b": {"a": 3, "bb": 22, "c": 44}}
+    node_props = _collect_op_props(pipeline)
+    assert node_props == exp
+
+    pipeline = compose(
+        "n", pipeline1, pipeline2, node_props={"bb": 22, "c": 44}, nest=True
+    )
     exp = {
         "n1.a": {"a": 1, "bb": 22, "c": 44},
         "n2.a": {"a": 11, "b": 0, "bb": 22, "c": 44},
         "n2.b": {"a": 3, "bb": 22, "c": 44},
     }
-    node_props = _collect_op_props(netop)
-    assert node_props == exp
-
-    netop = compose("n", netop1, netop2, node_props={"bb": 22, "c": 44}, nest=False)
-    exp = {"a": {"a": 1, "bb": 22, "c": 44}, "b": {"a": 3, "bb": 22, "c": 44}}
-    node_props = _collect_op_props(netop)
+    node_props = _collect_op_props(pipeline)
     assert node_props == exp
 
 
@@ -337,30 +341,43 @@ def test_as_renames(inp, exp):
 
 
 @pytest.mark.parametrize(
-    "op_kw, ex",
+    "prov_aliases, ex",
     [
         (
-            lambda: dict(aliases={"a": 1}),
-            r"The `aliases` for ['a'] rename ['a'], not found in provides []!",
+            ((), {"a": 1}),
+            TypeError(r"All `aliases` must be non-empty str, got: \[\('a', 1\)\]"),
         ),
         (
-            lambda: dict(name="t", provides="a", aliases={"a": 1, "b": 2}),
-            r"The `aliases` for ['a', 'b'] rename ['b'], not found in provides ['a']!",
+            ((), {"a": None}),
+            TypeError(r"All `aliases` must be non-empty str, got: \[\('a', None\)\]"),
         ),
         (
-            lambda: dict(name="t", provides=sfx("a"), aliases={sfx("a"): 1}),
-            "must not contain `sideffects",
+            ((), {"a": ""}),
+            TypeError(r"All `aliases` must be non-empty str, got: \[\('a', ''\)\]"),
         ),
         (
-            lambda: dict(name="t", provides="a", aliases={"a": sfx("AA")}),
-            "must not contain `sideffects",
+            ((), {"a": "A"}),
+            ValueError(
+                r"The `aliases` \['a'-->'A'\] rename non-existent provides in \[\]"
+            ),
+        ),
+        (
+            ("a", {"a": "A", "b": "B"}),
+            ValueError(
+                r"The `aliases` \['b'-->'B'\] rename non-existent provides in \['a'\]"
+            ),
+        ),
+        ((sfx("a"), {sfx("a"): "a"}), ValueError("must not contain `sideffects"),),
+        (("a", {"a": sfx("AA")}), ValueError("must not contain `sideffects"),),
+        (
+            (["a", "b"], {"a": "b"}),
+            ValueError(r"clash with existing provides in \['a', 'b'\]"),
         ),
     ],
 )
-def test_provides_aliases_BAD(op_kw, ex):
-    op_kw = op_kw()
-    with pytest.raises(ValueError, match=re.escape(ex)):
-        operation(str, **op_kw)()
+def test_func_op_validation_aliases_BAD(prov_aliases, ex):
+    with pytest.raises(type(ex), match=str(ex)):
+        reparse_operation_data("t", None, *prov_aliases)
 
 
 def test_provides_aliases():
@@ -400,47 +417,37 @@ def test_reschedule_unknown_dict_outs(caplog):
 
 def test_rescheduled_op_repr():
     op = operation(str, name="t", provides=["a"], rescheduled=True)
-    assert (
-        str(op) == "FunctionalOperation?(name='t', needs=[], provides=['a'], fn='str')"
-    )
+    assert str(op) == "FunctionalOperation?(name='t', provides=['a'], fn='str')"
 
 
 def test_endured_op_repr():
     op = operation(str, name="t", provides=["a"], endured=True)
-    assert (
-        str(op) == "FunctionalOperation!(name='t', needs=[], provides=['a'], fn='str')"
-    )
+    assert str(op) == "FunctionalOperation!(name='t', provides=['a'], fn='str')"
 
 
 def test_endured_rescheduled_op_repr():
     op = operation(str, name="t", rescheduled=1, endured=1)
-    assert str(op) == "FunctionalOperation!?(name='t', needs=[], provides=[], fn='str')"
+    assert str(op) == "FunctionalOperation!?(name='t', fn='str')"
 
 
 def test_parallel_op_repr():
     op = operation(str, name="t", provides=["a"], parallel=True)
-    assert (
-        str(op) == "FunctionalOperation|(name='t', needs=[], provides=['a'], fn='str')"
-    )
+    assert str(op) == "FunctionalOperation|(name='t', provides=['a'], fn='str')"
 
 
 def test_marshalled_op_repr():
     op = operation(str, name="t", provides=["a"], marshalled=True)
-    assert str(op) == (
-        "FunctionalOperation&(name='t', needs=[], provides=['a'], fn='str')"
-    )
+    assert str(op) == ("FunctionalOperation&(name='t', provides=['a'], fn='str')")
 
 
 def test_marshalled_parallel_op_repr():
     op = operation(str, name="t", parallel=1, marshalled=1)
-    assert str(op) == "FunctionalOperation|&(name='t', needs=[], provides=[], fn='str')"
+    assert str(op) == "FunctionalOperation|&(name='t', fn='str')"
 
 
 def test_ALL_op_repr():
     op = operation(str, name="t", rescheduled=1, endured=1, parallel=1, marshalled=1)
-    assert (
-        str(op) == "FunctionalOperation!?|&(name='t', needs=[], provides=[], fn='str')"
-    )
+    assert str(op) == "FunctionalOperation!?|&(name='t', fn='str')"
 
 
 def test_CONFIG_op_repr():
@@ -448,12 +455,9 @@ def test_CONFIG_op_repr():
         1
     ), tasks_marshalled(1):
         op = operation(str, name="t")
-        assert (
-            str(op)
-            == "FunctionalOperation!?|&(name='t', needs=[], provides=[], fn='str')"
-        )
+        assert str(op) == "FunctionalOperation!?|&(name='t', fn='str')"
     op = operation(str, name="t", rescheduled=0, endured=0, parallel=0, marshalled=0)
-    assert str(op) == "FunctionalOperation(name='t', needs=[], provides=[], fn='str')"
+    assert str(op) == "FunctionalOperation(name='t', fn='str')"
 
 
 def test_reschedule_outputs():
@@ -468,6 +472,10 @@ def test_reschedule_outputs():
 
     op = operation(lambda: NO_RESULT, name="t", provides=["a", "b"], rescheduled=True)
     assert op.compute({}) == {}
+    op = operation(
+        lambda: NO_RESULT_BUT_SFX, name="t", provides=["a", "b"], rescheduled=True
+    )
+    assert op.compute({}) == {}
 
     op = operation(
         lambda: {"b": "B"}, name="t", provides=["a", "b"], rescheduled=1, returns_dict=1
@@ -486,9 +494,9 @@ def test_reschedule_outputs():
 
 
 @pytest.mark.parametrize("attr, value", [("outputs", [1]), ("predicate", lambda: None)])
-def test_netop_narrow_attributes(attr, value):
-    netop = compose("1", operation(str, name="op1"))
-    assert getattr(netop.withset(**{attr: value}), attr) == value
+def test_pipeline_narrow_attributes(attr, value):
+    pipeline = compose("1", operation(str, name="op1"))
+    assert getattr(pipeline.withset(**{attr: value}), attr) == value
 
 
 _attr_values = [
@@ -522,7 +530,7 @@ def test_op_withset_conveys_attr(attr, value):
 
 
 @pytest.mark.parametrize("attr, value", _attr_values)
-def test_netop_conveys_attr_to_ops(attr, value):
+def test_pipeline_conveys_attr_to_ops(attr, value):
     def _opsattrs(ops, attr, value):
         vals = [getattr(op, attr) for op in ops if isinstance(op, Operation)]
         assert all(v == value for v in vals)
@@ -547,3 +555,75 @@ def test_netop_conveys_attr_to_ops(attr, value):
 )
 def test_dill_ops(op_fact):
     dill.loads(dill.dumps(op_fact()))
+
+
+def test_op_rename():
+    op = operation(
+        str, name="op1", needs=sfx("a"), provides=["a", sfx("b")], aliases=[("a", "b")],
+    )
+
+    def renamer(na):
+        assert na.op
+        assert not na.parent
+        return dep_renamed(na.name, lambda name: f"PP.{name}")
+
+    ren = op.withset(renamer=renamer)
+    got = str(ren)
+    assert got == (
+        """
+    FunctionalOperation(name='PP.op1', needs=[sfx('PP.a')], provides=['PP.a', sfx('PP.b')], aliases=[('PP.a', 'PP.b')], fn='str')
+        """.strip()
+    )
+
+
+def test_pipe_rename():
+    pipe = compose(
+        "t",
+        operation(str, name="op1", needs=sfx("a")),
+        operation(
+            str,
+            name="op2",
+            needs=sfx("a"),
+            provides=["a", sfx("b")],
+            aliases=[("a", "b")],
+        ),
+    )
+
+    def renamer(na):
+        assert na.op
+        assert not na.parent
+        return dep_renamed(na.name, lambda name: f"PP.{name}")
+
+    ren = pipe.withset(renamer=renamer)
+    got = str(ren)
+    assert got == (
+        """
+    Pipeline('t', needs=[sfx('PP.a')], provides=['PP.a', sfx('PP.b'), 'PP.b'], x2 ops: PP.op1, PP.op2)
+        """.strip()
+    )
+    got = str(ren.ops)
+    assert got == re.sub(
+        r"[\n ]+",  # collapse all space-chars into a single space
+        " ",
+        """
+        [FunctionalOperation(name='PP.op1', needs=[sfx('PP.a')], fn='str'),
+         FunctionalOperation(name='PP.op2', needs=[sfx('PP.a')], provides=['PP.a', sfx('PP.b')],
+         aliases=[('PP.a', 'PP.b')], fn='str')]
+        """.strip(),
+    )
+
+    ## Check dictionary with callables
+    #
+    ren = pipe.withset(
+        renamer={"op1": lambda n: "OP1", "op2": False, "a": optional("a"), "b": "B",}
+    )
+    got = str(ren.ops)
+    assert got == re.sub(
+        r"[\n ]+",  # collapse all space-chars into a single space
+        " ",
+        """
+        [FunctionalOperation(name='OP1', needs=[sfx('a')], fn='str'),
+         FunctionalOperation(name='op2', needs=[sfx('a')], provides=['a'(?), sfx('b')],
+         aliases=[('a'(?), 'B')], fn='str')]
+        """.strip(),
+    )
